MODULE Graph;
(* =========================================================================  
   Example RISC5 Oberon Module    

   Description:
     Map Slide Functions onto Astrobe Library functions 
  
   Target: Hardware-independent 

   (c) 2009-2019 CFB Software   
   http://www.astrobe.com  
   
  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)


IMPORT Display := ST7735, Graphics;

CONST
  size = 10;
  whiteOnBlack* = TRUE;
  MaxX* = Display.MaxX;
  MaxY* = Display.MaxY;

VAR
  highX*, highY*: INTEGER;
  colourMap: ARRAY 9 OF INTEGER;

PROCEDURE Init*;
BEGIN
  (* AutoCad colours are numbered differently to adapter colours. *)
  (* The colourMap array is used for conversion.                  *)
  colourMap[1] := Display.Red;
  colourMap[2] := Display.Yellow;
  colourMap[3] := Display.Green;
  colourMap[4] := Display.Cyan;
  colourMap[5] := Display.Blue;
  colourMap[6] := Display.Magenta;
  IF whiteOnBlack THEN
  	colourMap[7] := Display.White;
  	colourMap[8] := Display.Black
  ELSE
  	colourMap[7] := Display.Black;
  	colourMap[8] := Display.White
  END;

  highX := Display.MaxX;
  highY := Display.MaxY;
  Display.Init;
  Graphics.Init(Display.MaxX, Display.MaxY, Display.DrawDot)
END Init;

PROCEDURE Line*(fromX, fromY, toX, toY: INTEGER; colour: INTEGER);
BEGIN
  Graphics.Line(colourMap[colour], fromX, fromY, toX, toY)
END Line;

PROCEDURE ClearScreen*();
BEGIN
  Display.ClearScreen(Display.Black)
END ClearScreen;

END Graph.
