MODULE IO;
(* =========================================================================  
   Example RISC5 Oberon Module  

   Description:
   
     Functions to read AutoCad Slide data files  
  
     Ref: Project Oberon - Wirth & Gutknecht, ACM Press 1992

   Target: Hardware-independent

   (c) 2009-2019 CFB Software   
   http://www.astrobe.com  
   
  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

  IMPORT Files, SYSTEM;
  
	VAR
		b1, b2: INTEGER;
    F: Files.File;
    R: Files.Rider;
    offset: INTEGER;
		c: INTEGER;
    data: INTEGER;

  PROCEDURE ReadByte*(): BYTE;
  VAR
    byteNo: INTEGER;
    b: BYTE;
  BEGIN
    Files.ReadByte(R, b);
    RETURN b
  END ReadByte;


  PROCEDURE ReadWord(): INTEGER;
  VAR
    b1, b2: BYTE;
    i: INTEGER;
	BEGIN
		b1 := ReadByte();
    b2 := ReadByte();
		i := LSL(b2, 8) + b1;
    RETURN i
	END ReadWord;


	PROCEDURE ReadType* (): CHAR;
	BEGIN
		b1 := ReadByte();
		b2 := ReadByte();
		RETURN CHR(b2)
	END ReadType;


	PROCEDURE Offset(b: INTEGER): INTEGER;
  VAR
    offset: INTEGER;
  BEGIN
    IF b > 127 THEN offset := b - 256 ELSE offset := b END;
    RETURN offset
  END Offset;


  PROCEDURE GetOffset* (VAR o1, o2, o3, o4: INTEGER);
	BEGIN
		o1 := Offset(b1);
		o2 := Offset(ReadByte());
		o3 := Offset(ReadByte());
		o4 := Offset(ReadByte())
	END GetOffset;


	PROCEDURE GetColour* (): INTEGER;
	BEGIN
		RETURN b1
	END GetColour;


	PROCEDURE GetEndpoint* (VAR end1, end2: INTEGER);
	BEGIN
		end1 := Offset(b1);
		end2 := Offset(ReadByte())
	END GetEndpoint;


	PROCEDURE Get2Vector* (VAR v1, v2: INTEGER);
	BEGIN
		v1 := ReadWord();
		v2 := ReadWord();
		IF v2 < 0 THEN
			v2 := -v2;
		END
	END Get2Vector;


	PROCEDURE GetVector* (VAR v1, v2, v3, v4: INTEGER);
	BEGIN
		v1 := LSL(b2, 8) + b1;
		v2 := ReadWord();
		v3 := ReadWord();
		v4 := ReadWord()
	END GetVector;


	PROCEDURE ReadHeader* (VAR highX, highY: INTEGER);
  VAR
    i, b: INTEGER;
	BEGIN
    FOR i := 0 TO 18 DO b := ReadByte() END;
		highX := ReadWord();
		highY := ReadWord();
    FOR i := 23 TO 30 DO b := ReadByte() END
	END ReadHeader;


  PROCEDURE Reset*;
  BEGIN
    offset := 0;
    Files.Set(R, F, 0) 
  END Reset;
  
  
  PROCEDURE Open*(name: ARRAY OF CHAR): BOOLEAN;
  VAR 
    ok: BOOLEAN;
	BEGIN
    ok := FALSE;
    F := Files.Old(name);
    IF F # NIL THEN
      ok := TRUE;
      Files.Set(R, F, 0) 
    END;
    RETURN ok
	END Open;

END IO.
