MODULE Slide;
(* =========================================================================  
   Example RISC5 Oberon Module  

   Description:
     Display an AutoCad Slide-format File 
  
   Target: Hardware-independent 

   (c) 2009-2019 CFB Software   
   http://www.astrobe.com  
   
  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

  IMPORT Graph, IO;

  CONST
    offset    = 0FBX; (* -5 *)
    endfile   = 0FCX; (* -4 *)
    endPoint  = 0FEX; (* -2 *)
    newColour = 0FFX; (* -1 *)

  VAR
    c: CHAR;
    colour: INTEGER;
    scaleFactor: REAL;
    hX, hY: INTEGER;
    panX, panY: INTEGER;
    Init*: PROCEDURE();

  PROCEDURE XScale (x: INTEGER): INTEGER;
  BEGIN
    RETURN FLOOR(FLT(x) * scaleFactor) - panX
  END XScale;

  PROCEDURE YScale (y: INTEGER): INTEGER;
  BEGIN
    RETURN Graph.MaxY - FLOOR(FLT(y) * scaleFactor) - panY  
  END YScale;

  PROCEDURE Draw*(zoom: REAL; px, py: INTEGER);
  VAR
    drawLine: BOOLEAN;
    end1, end2, fromX, fromY, toX, toY: INTEGER;
    lastX, lastY: INTEGER;
    offset1, offset2, offset3, offset4: INTEGER;
  BEGIN
    Graph.ClearScreen();
    IO.Reset();
    IO.ReadHeader(hX, hY);
    IF hX > hY THEN 
      scaleFactor := zoom * FLT(Graph.MaxX) / FLT(hX)
    ELSE 
      scaleFactor := zoom * FLT(Graph.MaxY) / FLT(hY) 
    END;
    panX := px;
    panY := py;
    c := IO.ReadType();
    lastX := 0;
    lastY := 0;
    REPEAT
      drawLine := TRUE;
      IF c = offset THEN 
        IO.GetOffset(offset1, offset2, offset3, offset4);
        fromX := lastX + offset1;
        fromY := lastY + offset2;
        toX := lastX + offset3;
        toY := lastY + offset4;
        lastX := fromX;
        lastY := fromY
      ELSIF c = endPoint THEN 
        fromX := lastX;
        fromY := lastY;
        IO.GetEndpoint(end1, end2);
        toX := lastX + end1;
        toY := lastY + end2;
        lastX := toX;
        lastY := toY
      ELSIF c = newColour THEN 
        colour := IO.GetColour();
        drawLine := FALSE
      ELSE
        IO.GetVector(fromX, fromY, toX, toY);
        lastX := fromX;
        lastY := fromY
      END;
      IF drawLine THEN
        Graph.Line(XScale(fromX), YScale(fromY), XScale(toX), YScale(toY), colour)
      END;
      c := IO.ReadType()
    UNTIL (c = endfile)
  END Draw;

BEGIN
  Init := Graph.Init
END Slide.
